// Autor generatoru: Václav Volhejn

#include <iostream>
#include <random>
#include <vector>
#include <cassert>
#include <unordered_set>
#include <algorithm>
#include <iomanip>
#include <fstream>
#include <sstream>

// rand() nepouzivame, napr. protoze neni definovane, jak velka cisla muze vracet
#define rand(x) ("Misto rand() prosim pouzivej funkce rand* definovane zde", +)
#define ll long long
using namespace std;

#define rep(i, a, n) for (int i = a; i < (int)(n); i++)

// Generator nahodnych cisel
std::mt19937_64 rng;

ll randRange(ll from, ll to) {
    // Vrati nahodny long long v intervalu [from, to] (tj. vcetne `to`).
    std::uniform_int_distribution<ll> dist(from, to);
    return dist(rng);
}

double randDouble(double from, double to) {
    // Vrati nahodny double v intervalu [from, to), (tj. bez `to`).
    std::uniform_real_distribution<double> dist(from, to);
    return dist(rng);
}

ll randAlmost(ll to, double min_part = 0.9) {
    // Vrati nahodny long long ktery je v nejakem smyslu "skoro tak velky" jako `to`,
    // konkretne vrati cislo v intervalu [floor(value*min_part), value].
    // Uzitecne napr. pro generovani velikosti problemu
    assert(to >= 1);
    return randRange(min_part * to, to);
}

bool randBool(double p = 0.5) {
    // Vrati true s pravdepodobnosti p a false s pravdepodobnosti 1-p
    return randDouble(0, 1) <= p;
}

vector<ll> randDifferentNumbers(ll from, ll to, int k) {
    // Vrati k ruznych nahodnych cisel z intervalu [from, to]
    // Implementace podle: https://www.gormanalysis.com/blog/random-numbers-in-cpp/

    unordered_set<ll> samples;
    ll n = to - from + 1;
    assert(n >= k);

    for (ll r = n - k; r < n; ++r) {
        ll v = uniform_int_distribution<ll>(from, from + r)(rng);
        if (!samples.insert(v).second) samples.insert(from + r);
    }

    vector<ll> result(samples.begin(), samples.end());
    shuffle(result.begin(), result.end(), rng);

    return result;
}


// ---------------------------------------------------------------------------
// TODO: upravit tuto ukazku pro svou ulohu

const int DIFFS = 4;
const int MIN_N = 2;
// Pro konstanty, ktere se lisi mezi obtiznostmi, muzeme pouzit takovyto format;
// MAX_N[diff] rika, jake je maximalni N pro danou obtiznost (0 nebo 1)
const int MAX_N[DIFFS] = {2, 500, 500, int(1e6)};
const unordered_set<int> ALLOWED[DIFFS] = {
    { -1, 0, 1},
    {0, 1},
    { -1, 0, 1},
    { -1, 0, 1},
};

struct Problem {
    // Reprezentuje jeden problem (test).
    vector<int> a;

    Problem() {}

    void check(int diff) {
        // Tato metoda neni nutna, ale je uzitecna jako zakladni kontrola:
        // zkontroluje, jestli problem odpovida limitum stanovenym v zadani
        int n = a.size();
        assert(MIN_N <= n && n <= MAX_N[diff]);
        for (int i = 0; i < n; i++) {
            assert(ALLOWED[diff].count(a[i]));
        }
    }

    void print(ostream& out) {
        // Vypise problem tak, jak to specifikuje sekce "Tvar vstupu"
        out << a.size() << endl;
        for (int i = 0; i < a.size(); i++) {
            if (i > 0) {
                out << " ";
            }
            out << a[i];
        }
        out << endl;
    }
};

// Zde je nekolik funkci na vygenerovani vstupu konkretniho druhu. Nejsou
// samozrejme aplikovatelne na jine ulohy, ale obecne se zamyslete, jake
// zvlastni pripady mohou napsat a jake potencialni chyby nebo spatna reseni
// by mohl resitel udelat. Vzdy by mel existovat test, ktery tuto chybu
// detekuje a reseni nenecha projit. Take by mel existovat test (nebo testy),
// kde vstup je opravdu tak velky, jak to dovoluji limity v zadani,
// aby neprosla pomala reseni

Problem genRandomWalk(int n, unordered_set<int> allowed) {
    vector<int> allowed_v;
    for (int x : allowed) {
        allowed_v.push_back(x);
    }

    Problem problem;
    for (int i = 0; i < n; i++) {
        problem.a.push_back(allowed_v[randRange(0, allowed.size() - 1)]);
    }
    return problem;
}

int main(int argc, char* argv[]) {
    if (argc != 2) {
        cerr << "pouziti: ./gen slozka_pro_vygenerovane_vstupy" << endl;
        exit(1);
    }
    string dir(argv[1]);

    rng.seed(125);

    rep(diff, 0, DIFFS) {
        vector<Problem> problems;
        rep(i, 0, 3) {
            int n = max(MIN_N, (int) randAlmost(MAX_N[diff]));
            problems.push_back(genRandomWalk(n, ALLOWED[diff]));
        }
        problems.push_back(genRandomWalk(MAX_N[diff], ALLOWED[diff]));

        unordered_set<int> withoutZero;
        for (int x : ALLOWED[diff]) {
            if (x != 0) withoutZero.insert(x);
        }
        problems.push_back(genRandomWalk(MAX_N[diff], withoutZero));

        rep(i, 0, problems.size()) {
            Problem& p = problems[i];
            p.check(diff);

            stringstream filename;
            filename << dir << "/"
                     << setw(2) << setfill('0') << (diff + 1)
		     << (char)('a' + i)
                     << ".in";

            ofstream file;
            file.open(filename.str());
            p.print(file);
            file.close();
        }
    }
}
